*Gas heating system with 2 storages (buffer storage and DHW tank); gas heating is non modulating; 2 additional modulating electric heating elements
*Decentralized optimization; Building type 3 in the paper "Decentralized optimization approaches for using the load flexibility of electric heating devices"
*Generate Diverse solution by maximizing difference to a given solution (total electrical power profile)
* with an vector r indicating which timeslots should be considered for diversity. Here the total difference between two profiles is maximized.


*@author: Thomas Dengiz
*Chair of Energy Economics, Karlsruhe Institute of Technology (KIT) in Germany
*Webpage: https://www.iip.kit.edu/english/86_3459.php
*E-Mail: thomas.dengiz@kit.edu


*Define the sets: t_a is the index for the timeslots and household_mGas is the index of the building
Set
    t_a timeslots
    household_mGas household_mGas


*Read input Data from gdx-file
$GDXIN 'GDXDataForDiversity.gdx';
$LOAD t_a
$LOAD household_mGas

*Set up time markers for measuring the execution time
Scalar time_comp, time_exec, time_elapsed;


*Definition of the parameters and reading from gdx-file

Parameter
demand_heating_mGas(t_a, household_mGas)
$LOAD  demand_heating_mGas

demand_electrical_mGas(t_a, household_mGas)
$LOAD  demand_electrical_mGas

volumeOfBufferTank_mGas (household_mGas)
$LOAD volumeOfBufferTank_mGas

volumeOfDHWTank_mGas (household_mGas)
$LOAD volumeOfDHWTank_mGas

electricalPower_SH_mGas (household_mGas)
$LOAD electricalPower_SH_mGas

electricalPower_DHW_mGas (household_mGas)
$LOAD electricalPower_DHW_mGas

gasPower (household_mGas)
$LOAD gasPower


standingLoss_BufferTank_mGas (household_mGas)
$LOAD standingLoss_BufferTank_mGas

standingLoss_DHWTank_mGas (household_mGas)
$LOAD standingLoss_DHWTank_mGas

demand_DHW_mGas(t_a, household_mGas)
$LOAD demand_DHW_mGas

pv_Nominal_mGas(t_a, household_mGas)
$LOAD pv_Nominal_mGas

pv_Peak_mGas (household_mGas)
$LOAD pv_Peak_mGas

initalBufferStorageTemperature_mGas (household_mGas)
$LOAD initalBufferStorageTemperature_mGas

initalDHWTankTemperature_mGas (household_mGas)
$LOAD initalDHWTankTemperature_mGas

optimalValue_kWh
$LOAD optimalValue_kWh

deviationPercentageFromOptimum
$LOAD deviationPercentageFromOptimum

electricalPowerTotalOptimal (t_a)
$LOAD electricalPowerTotalOptimal

startingTimeSlotDiversity
$LOAD startingTimeSlotDiversity

endingTimeSlotDiversity
$LOAD endingTimeSlotDiversity

r (t_a)
$LOAD r

option_relativeGap
$LOAD option_relativeGap

option_absoluteGap
$LOAD option_absoluteGap

option_timeLimit
$LOAD option_timeLimit

resultDiversity_in_kWH





*Definition of the variables
positive variable  xElectric_mGas(t_a, household_mGas) heatGenerationForBufferTank;
positive variable yElectric_mGas(t_a, household_mGas) heatGenerationForDHWTank;
binary variable  x_mGas(t_a, household_mGas) heatGenerationForBufferTank;
binary variable y_mGas(t_a, household_mGas) heatGenerationForDHWTank;
binary variable xAct_mGas(t_a, household_mGas);

variable
temperature_level_bufferTank_mGas (t_a, household_mGas)
temperature_level_DHWTank_mGas (t_a, household_mGas);



Variables
         z surplusPowerTotal
         z_correct correctedSurplusPower
         d diversity;


variable surplusPower(t_a);
positive variable positiveSurplusPower(t_a);
positive variable negativeSurplusPower(t_a);
binary variable isPositive(t_a);
positive variable pvGenerationTotal(t_a);
positive variable electricalPowerTotal(t_a);

variable electricalPowerTotalDifference(t_a);
positive variable positiveElectricalPowerTotalDifference (t_a);
positive variable negativeElectricalPowerTotalDifference (t_a);
binary variable isPositiveElectricalPowerTotalDifference (t_a);


*Definition of the scalar values
scalar
    minimalBufferTankTemperature_mGas /30/
    maximalBufferTankTemperature_mGas /45/
    minimalDHWTankTemperature_mGas /10/
    maximalDHWTankTemperature_mGas /45/
    efficiencyElectricHeating /0.98/
    efficiencyGasHeating /0.98/
    DensityOfWater / 988.02/
    SpecificHeatCapacityOfWater /4181/
    timeSlotForEndTemperature /287/
    timeResolution / 300/;
       



temperature_level_DHWTank_mGas.lo(t_a, household_mGas) = minimalDHWTankTemperature_mGas;
temperature_level_DHWTank_mGas.up(t_a, household_mGas) = maximalDHWTankTemperature_mGas;

temperature_level_bufferTank_mGas.lo(t_a, household_mGas) = minimalBufferTankTemperature_mGas;
temperature_level_bufferTank_mGas.up(t_a, household_mGas) = maximalBufferTankTemperature_mGas;

temperature_level_DHWTank_mGas.fx("1", household_mGas) = initalDHWTankTemperature_mGas (household_mGas);
temperature_level_bufferTank_mGas.fx("1", household_mGas) = initalBufferStorageTemperature_mGas(household_mGas);




*Definition of the equations
Equations
eq_mGas_temperature_level_bufferTank_mGas(t_a, household_mGas)     hot water tank temperature
eq_mGas_temperature_level_DHWTank_mGas(t_a, household_mGas)     hot water tank temperature
eq_mGas_onlyOneTankAtOneTime (t_a, household_mGas)         
eq_mGas_onlyXorY_1 (t_a, household_mGas)
eq_mGas_onlyXorY_2 (t_a, household_mGas)
eq_mGas_minimalEndTemperature_bufferTank(t_a, household_mGas)
eq_mGas_maximalEndTemperature_bufferTank (t_a, household_mGas)
eq_mGas_minimalEndTemperature_dhwTank(t_a, household_mGas)
eq_mGas_maximalEndTemperature_dhwTank (t_a, household_mGas)
eq_pvGenerationTotal(t_a)
eq_surplusPower(t_a)
eq_electricalPowerTotal(t_a)
e1(t_a), e2(t_a) ,e3(t_a)
          eq_diversity
          eq_electricalPowerTotalDifference(t_a)
          eq_electricalPowerTotalDifference_e1(t_a)
          eq_electricalPowerTotalDifference_e2(t_a)
          eq_electricalPowerTotalDifference_e3(t_a)
          eq_fobj_help
          eq_fobj_deviation
;


*Constraints for the minimal and maximal temperature at the end of the optimization horizon

eq_mGas_minimalEndTemperature_bufferTank(t_a, household_mGas) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_mGas (t_a,household_mGas)
=g=
minimalBufferTankTemperature_mGas + 6;


eq_mGas_maximalEndTemperature_bufferTank(t_a, household_mGas) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_mGas (t_a,household_mGas)
 =l=
 maximalBufferTankTemperature_mGas - 6;


eq_mGas_minimalEndTemperature_dhwTank(t_a, household_mGas) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_mGas (t_a,household_mGas)
=g=
minimalDHWTankTemperature_mGas + 15;


eq_mGas_maximalEndTemperature_dhwTank(t_a, household_mGas) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_mGas (t_a,household_mGas)
 =l=
 maximalDHWTankTemperature_mGas - 15;


*Definition of the energetic difference equation for the temperature of the buffer storage
 eq_mGas_temperature_level_bufferTank_mGas(t_a, household_mGas) $(ord(t_a)>1)..
temperature_level_bufferTank_mGas(t_a, household_mGas)
=e=
temperature_level_bufferTank_mGas (t_a-1, household_mGas) + ((x_mGas(t_a, household_mGas) *  gasPower (household_mGas) * efficiencyGasHeating * timeResolution  + xElectric_mGas(t_a, household_mGas) * electricalPower_SH_mGas(household_mGas) * timeResolution * efficiencyElectricHeating - demand_heating_mGas(t_a, household_mGas)* timeResolution  - standingLoss_BufferTank_mGas(household_mGas) * timeResolution) /((volumeOfBufferTank_mGas(household_mGas)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_mGas_temperature_level_DHWTank_mGas(t_a, household_mGas) $(ord(t_a)>1)..
temperature_level_DHWTank_mGas(t_a, household_mGas)
=e=
temperature_level_DHWTank_mGas (t_a-1, household_mGas) + ((y_mGas(t_a, household_mGas) *  gasPower (household_mGas) * efficiencyGasHeating * timeResolution  + yElectric_mGas(t_a, household_mGas) * electricalPower_DHW_mGas(household_mGas) * timeResolution * efficiencyElectricHeating     - demand_DHW_mGas(t_a, household_mGas) * timeResolution - standingLoss_DHWTank_mGas(household_mGas) * timeResolution) /((volumeOfDHWTank_mGas(household_mGas)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));
 


*Constraints ensure that only one storage is heated up in each time slot by the gas heating device
eq_mGas_onlyOneTankAtOneTime(t_a, household_mGas)..
y_mGas(t_a, household_mGas) + x_mGas(t_a, household_mGas) =l= 1;
eq_mGas_onlyXorY_1(t_a,household_mGas).. x_mGas(t_a,household_mGas)    =l= xAct_mGas(t_a,household_mGas);    
eq_mGas_onlyXorY_2(t_a,household_mGas).. y_mGas(t_a,household_mGas)    =l= (1-xAct_mGas(t_a,household_mGas));



*Help equations to first caluculate the Surplus power and ensure that the surplus energy of the new profile only differs to a certain degree from the optimal value for the surplus energy of the previous optimization problem ("Surplus Energy Minimization Problem")
eq_fobj_help ..
z
=e=
sum(t_a,positiveSurplusPower(t_a));
eq_fobj_deviation ..
z =l= optimalValue_kWh *3600000/timeResolution + (optimalValue_kWh *(3600000/timeResolution) * ((deviationPercentageFromOptimum -0)/100));


*Objective function for maximizing the total differnce form the output load profile of the  previous optimization problem ("Surplus Energy Minimization Problem")
eq_diversity..
d
=e=
sum((t_a  )  ,(negativeElectricalPowerTotalDifference(t_a)+ positiveElectricalPowerTotalDifference(t_a)) * r(t_a) )
- sum((t_a, household_mGas), 0.01* (xElectric_mGas(t_a, household_mGas)* electricalPower_SH_mGas(household_mGas) + yElectric_mGas(t_a, household_mGas) * electricalPower_DHW_mGas(household_mGas)));


*Big_M approach for splitting up the difference in the load profiles into a positive and a negative part. Thus it is possible to maximize the absolute difference between the new load profile and the output load profile of the  previous optimization problem ("Surplus Energy Minimization Problem")
parameter       
       bigMpos(t_a)
       bigMneg(t_a)  ;

eq_electricalPowerTotalDifference(t_a)..
electricalPowerTotalDifference(t_a) =e= electricalPowerTotalOptimal(t_a) - electricalPowerTotal(t_a);

eq_electricalPowerTotalDifference_e1(t_a)..
electricalPowerTotalDifference(t_a) =e= positiveElectricalPowerTotalDifference(t_a) - negativeElectricalPowerTotalDifference(t_a);

eq_electricalPowerTotalDifference_e2(t_a)..
positiveElectricalPowerTotalDifference(t_a) =l= isPositiveElectricalPowerTotalDifference(t_a) *  bigMneg(t_a) ;

eq_electricalPowerTotalDifference_e3(t_a)..
negativeElectricalPowerTotalDifference(t_a) =l=  (1-isPositiveElectricalPowerTotalDifference(t_a)) * bigMneg(t_a) ;


* Maximal values for the decision variables xElectric and yElectric is 1.0
xElectric_mGas.up(t_a, household_mGas) = 1;
yElectric_mGas.up(t_a, household_mGas) = 1;


*Equations for calculating the total PV generation, the surplus power and the total electrical power
eq_pvGenerationTotal(t_a).. pvGenerationTotal(t_a) =e= sum(household_mGas, (pv_Nominal_mGas(t_a, household_mGas) * pv_Peak_mGas(household_mGas)));
eq_surplusPower(t_a).. surplusPower(t_a) =e= pvGenerationTotal(t_a) - electricalPowerTotal(t_a);
eq_electricalPowerTotal(t_a)..electricalPowerTotal(t_a) =e= sum(household_mGas, (xElectric_mGas(t_a, household_mGas)* electricalPower_SH_mGas(household_mGas) + yElectric_mGas(t_a, household_mGas) * electricalPower_DHW_mGas(household_mGas) + demand_electrical_mGas(t_a, household_mGas)));


*Big-M approach for ensuring that only the positive surplus power is minimized 
e1(t_a).. surplusPower(t_a) =e= positiveSurplusPower(t_a) - negativeSurplusPower(t_a);
e2(t_a).. positiveSurplusPower(t_a) =l= bigMpos(t_a) * isPositive(t_a);
e3(t_a).. negativeSurplusPower(t_a) =l= bigMneg(t_a) * (1-isPositive(t_a));

bigMpos(t_a) = sum(household_mGas, (pv_Nominal_mGas(t_a, household_mGas) * pv_Peak_mGas(household_mGas)));
bigMneg(t_a) = sum(household_mGas, (electricalPower_SH_mGas(household_mGas) + electricalPower_DHW_mGas(household_mGas) + demand_electrical_mGas(t_a, household_mGas)));






*Defintion of the model that includes all equations
Model
     heating_scheduling /all/ ;



*Specifying the MIP gaps for the optimizations
heating_scheduling.optcr=option_relativeGap;
heating_scheduling.optca=option_absoluteGap;
heating_scheduling.Reslim=option_timeLimit;





Solve heating_scheduling using mip maximizing d;

*Calculating the time for the optimization
time_comp = TimeComp;
time_exec = TimeExec;
time_elapsed = TimeElapsed;


*Converting the sum of diversity power into energy in kWh
resultDiversity_in_kWH   = d.L * timeResolution / 3600000 ;



