*Heating system with a non-modulating heat pump and two storages
*Decentralized optimization; Building type 1 in the paper "Decentralized optimization approaches for using the load flexibility of electric heating devices"
*

*@author: Thomas Dengiz
*Chair of Energy Economics, Karlsruhe Institute of Technology (KIT) in Germany
*Webpage: https://www.iip.kit.edu/english/86_3459.php
*E-Mail: thomas.dengiz@kit.edu




*Define the sets: t_a is the index for the timeslots and household_HP is the index of the building
Set
    t_a timeslots
    household_HP household_HP



*Read input Data from gdx-file
$GDXIN 'InputGDXdataSingleOpt.gdx';
$LOAD t_a
$LOAD household_HP

*Set up time markers for measuring the execution time
Scalar time_comp, time_exec, time_elapsed;





*Definition of the parameters and reading from gdx-file

Parameter
demand_heating_HP(t_a, household_HP)
$LOAD  demand_heating_HP

demand_electrical_HP(t_a, household_HP)
$LOAD  demand_electrical_HP

volumeOfBufferTank_HP (household_HP)
$LOAD volumeOfBufferTank_HP

volumeOfDHWTank_HP (household_HP)
$LOAD volumeOfDHWTank_HP

electricalPower_HP (household_HP)
$LOAD electricalPower_HP

standingLoss_BufferTank_HP (household_HP)
$LOAD standingLoss_BufferTank_HP

standingLoss_DHWTank_HP (household_HP)
$LOAD standingLoss_DHWTank_HP

demand_DHW_HP(t_a, household_HP)
$LOAD demand_DHW_HP

pv_Nominal_HP(t_a, household_HP)
$LOAD pv_Nominal_HP

pv_Peak_HP (household_HP)
$LOAD pv_Peak_HP

initalBufferStorageTemperature_HP (household_HP)
$LOAD initalBufferStorageTemperature_HP

initalDHWTankTemperature_HP (household_HP)
$LOAD initalDHWTankTemperature_HP

cop_BufferTank_HP(t_a, household_HP)
$LOAD cop_BufferTank_HP

cop_DHWTank_HP(t_a, household_HP)
$LOAD cop_DHWTank_HP

option_relativeGap
$LOAD option_relativeGap

option_absoluteGap
$LOAD option_absoluteGap

option_timeLimit
$LOAD option_timeLimit



result_in_kWH 





*Definition of the variables

Variables
         z surplusPowerTotal;



variable surplusPower(t_a);
positive variable positiveSurplusPower(t_a);
positive variable negativeSurplusPower(t_a);
binary variable isPositive(t_a);
positive variable pvGenerationTotal(t_a);
positive variable electricalPowerTotal(t_a);

binary variable  x_HP(t_a, household_HP) heatGenerationForBufferTank;
binary variable y_HP(t_a, household_HP) heatGenerationForDHWTank;
binary variable xAct_HP(t_a, household_HP);

variable
temperature_level_bufferTank_HP (t_a, household_HP)
temperature_level_DHWTank_HP (t_a, household_HP);


*Definition of the scalar values

scalar
    minimalBufferTankTemperature_HP /20/
    maximalBufferTankTemperature_HP /22/
    minimalDHWTankTemperature_HP /10/
    maximalDHWTankTemperature_HP /45/
    efficiencyElectricHeating /0.98/
    efficiencyGasHeating /0.98/
    DensityOfWater / 988.02/
    DensityOfCement /2400/
    SpecificHeatCapacityOfCement /1000/
    SpecificHeatCapacityOfWater /4181/
    timeSlotForEndTemperature /287/
    timeResolution / 300/;
       

*Setting upper and lower limits for the temperature levels of the storages
temperature_level_DHWTank_HP.lo(t_a, household_HP) = minimalDHWTankTemperature_HP;
temperature_level_DHWTank_HP.up(t_a, household_HP) = maximalDHWTankTemperature_HP;

temperature_level_bufferTank_HP.lo(t_a, household_HP) = minimalBufferTankTemperature_HP;
temperature_level_bufferTank_HP.up(t_a, household_HP) = maximalBufferTankTemperature_HP;


*Fixing the inital temperature levels of the storages
temperature_level_DHWTank_HP.fx("1", household_HP) = initalDHWTankTemperature_HP (household_HP);
temperature_level_bufferTank_HP.fx("1", household_HP) = initalBufferStorageTemperature_HP(household_HP);



*Definition of the equations

Equations
eq_HP_temperature_level_bufferTank_HP(t_a, household_HP)     hot water tank temperature
eq_HP_temperature_level_DHWTank_HP(t_a, household_HP)     hot water tank temperature
eq_HP_onlyOneTankAtOneTime (t_a, household_HP)         
eq_HP_onlyXorY_1 (t_a, household_HP)
eq_HP_onlyXorY_2 (t_a, household_HP)
eq_fobj
eq_HP_minimalEndTemperature_bufferTank(t_a, household_HP)
eq_HP_maximalEndTemperature_bufferTank (t_a, household_HP)
eq_HP_minimalEndTemperature_dhwTank(t_a, household_HP)
eq_HP_maximalEndTemperature_dhwTank (t_a, household_HP)
eq_pvGenerationTotal(t_a)
eq_surplusPower(t_a)
eq_electricalPowerTotal(t_a)
e1(t_a), e2(t_a) ,e3(t_a)
;



*Constraints for the minimal and maximal temperature at the end of the optimization horizon

eq_HP_minimalEndTemperature_bufferTank(t_a, household_HP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_HP (t_a,household_HP)
=g=
minimalBufferTankTemperature_HP + 0.9;


eq_HP_maximalEndTemperature_bufferTank(t_a, household_HP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_HP (t_a,household_HP)
 =l=
 maximalBufferTankTemperature_HP - 0.9;


eq_HP_minimalEndTemperature_dhwTank(t_a, household_HP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_HP (t_a,household_HP)
=g=
minimalDHWTankTemperature_HP + 15;


eq_HP_maximalEndTemperature_dhwTank(t_a, household_HP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_HP (t_a,household_HP)
 =l=
 maximalDHWTankTemperature_HP - 15;


*Definition of the energetic difference equation for the temperature of the buffer storage 
eq_HP_temperature_level_bufferTank_HP(t_a, household_HP) $(ord(t_a)>1)..
temperature_level_bufferTank_HP(t_a, household_HP)
=e=
temperature_level_bufferTank_HP(t_a-1, household_HP) + ((x_HP(t_a, household_HP) * electricalPower_HP(household_HP) * timeResolution *  cop_BufferTank_HP(t_a, household_HP) - demand_heating_HP(t_a, household_HP)* timeResolution  - standingLoss_BufferTank_HP(household_HP) * timeResolution) /((volumeOfBufferTank_HP(household_HP)/1000) * DensityOfCement * SpecificHeatCapacityOfCement));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_HP_temperature_level_DHWTank_HP(t_a, household_HP) $(ord(t_a)>1)..
temperature_level_DHWTank_HP(t_a, household_HP)
=e=
temperature_level_DHWTank_HP (t_a-1, household_HP) + ((y_HP(t_a, household_HP) * electricalPower_HP(household_HP) * timeResolution * cop_DHWTank_HP(t_a, household_HP)     - demand_DHW_HP(t_a, household_HP) * timeResolution - standingLoss_DHWTank_HP(household_HP) * timeResolution) /((volumeOfDHWTank_HP(household_HP)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));




*Constraints ensure that only one storage is heated up in each time slot
eq_HP_onlyOneTankAtOneTime(t_a, household_HP)..
y_HP(t_a, household_HP) + x_HP(t_a, household_HP) =l= 1;
eq_HP_onlyXorY_1(t_a,household_HP).. x_HP(t_a,household_HP)    =l= xAct_HP(t_a,household_HP);    
eq_HP_onlyXorY_2(t_a,household_HP).. y_HP(t_a,household_HP)    =l= (1-xAct_HP(t_a,household_HP));




* Maximal values for the decision variables x and y is 1.0
x_HP.up(t_a, household_HP) = 1;
y_HP.up(t_a, household_HP) = 1;



*Equations for calculating the total PV generation, the surplus power and the total electrical power
eq_pvGenerationTotal(t_a).. pvGenerationTotal(t_a) =e= sum(household_HP, (pv_Nominal_HP(t_a, household_HP) * pv_Peak_HP(household_HP)));
eq_surplusPower(t_a).. surplusPower(t_a) =e= pvGenerationTotal(t_a) - electricalPowerTotal(t_a);
eq_electricalPowerTotal(t_a)..electricalPowerTotal(t_a) =e= sum(household_HP, (x_HP(t_a, household_HP)* electricalPower_HP(household_HP) + y_HP(t_a,household_HP) * electricalPower_HP(household_HP) + demand_electrical_HP(t_a, household_HP)));


*Big-M approach for ensuring that only the positive surplus power is minimized 

parameter       
       bigMpos(t_a)
       bigMneg(t_a) ; 

e1(t_a).. surplusPower(t_a) =e= positiveSurplusPower(t_a) - negativeSurplusPower(t_a);
e2(t_a).. positiveSurplusPower(t_a) =l= bigMpos(t_a) * isPositive(t_a);
e3(t_a).. negativeSurplusPower(t_a) =l= bigMneg(t_a) * (1-isPositive(t_a));

bigMpos(t_a) = sum(household_HP, (pv_Nominal_HP(t_a, household_HP) * pv_Peak_HP(household_HP)));
bigMneg(t_a) = sum(household_HP, (electricalPower_HP(household_HP) + electricalPower_HP(household_HP) + demand_electrical_HP(t_a, household_HP)));



*Objective function: minimize sum of surplus power over every time slot --> minimize surplus energy

eq_fobj ..
z
=e=
sum((t_a), positiveSurplusPower(t_a) );



*Defintion of the model that includes all equations
Model
     heating_scheduling /all/ ;


*Specifying the MIP gaps for the optimizations
heating_scheduling.optcr=option_relativeGap;
heating_scheduling.optca=option_absoluteGap;
heating_scheduling.Reslim=option_timeLimit;





Solve heating_scheduling using mip minimizing z;

*Calculating the time for the optimization
time_comp = TimeComp;
time_exec = TimeExec;
time_elapsed = TimeElapsed;

*Converting the sum of surplus power into surplus energy in kWh
result_in_kWh   = z.L * timeResolution / 3600000 ;

