*Heating system with a modulating heat pump and two storages
*Decentralized optimization; Building type 2 in the paper "Decentralized optimization approaches for using the load flexibility of electric heating devices"


*@author: Thomas Dengiz
*Chair of Energy Economics, Karlsruhe Institute of Technology (KIT) in Germany
*Webpage: https://www.iip.kit.edu/english/86_3459.php
*E-Mail: thomas.dengiz@kit.edu


*Define the sets: t_a is the index for the timeslots and household_mHP is the index of the building
Set
    t_a timeslots
    household_mHP household_mHP


*Read input Data from gdx-file
$GDXIN 'InputGDXdataSingleOpt.gdx';
$LOAD t_a
$LOAD household_mHP

*Set up time markers for measuring the execution time
 Scalar time_comp, time_exec, time_elapsed;
time_comp = TimeComp;
time_exec = TimeExec;
time_elapsed = TimeElapsed;



*Definition of the parameters and reading from gdx-file

Parameter
demand_heating_mHP(t_a, household_mHP)
$LOAD  demand_heating_mHP

demand_electrical_mHP(t_a, household_mHP)
$LOAD  demand_electrical_mHP

volumeOfBufferTank_mHP (household_mHP)
$LOAD volumeOfBufferTank_mHP

volumeOfDHWTank_mHP (household_mHP)
$LOAD volumeOfDHWTank_mHP

electricalPower_mHP (household_mHP)
$LOAD electricalPower_mHP

standingLoss_BufferTank_mHP (household_mHP)
$LOAD standingLoss_BufferTank_mHP

standingLoss_DHWTank_mHP (household_mHP)
$LOAD standingLoss_DHWTank_mHP

demand_DHW_mHP(t_a, household_mHP)
$LOAD demand_DHW_mHP

pv_Nominal_mHP(t_a, household_mHP)
$LOAD pv_Nominal_mHP

pv_Peak_mHP (household_mHP)
$LOAD pv_Peak_mHP

initalBufferStorageTemperature_mHP (household_mHP)
$LOAD initalBufferStorageTemperature_mHP

initalDHWTankTemperature_mHP (household_mHP)
$LOAD initalDHWTankTemperature_mHP

cop_BufferTank_mHP(t_a, household_mHP)
$LOAD cop_BufferTank_mHP

cop_DHWTank_mHP(t_a, household_mHP)
$LOAD cop_DHWTank_mHP

option_relativeGap
$LOAD option_relativeGap

option_absoluteGap
$LOAD option_absoluteGap

option_timeLimit
$LOAD option_timeLimit



result_in_kWH 


*Definition of the variables

positive variable  x_mHP(t_a, household_mHP) heatGenerationForBufferTank;
positive variable y_mHP(t_a, household_mHP) heatGenerationForDHWTank;
binary variable xAct_mHP(t_a, household_mHP);

variable
temperature_level_bufferTank_mHP (t_a, household_mHP)
temperature_level_DHWTank_mHP (t_a, household_mHP);



Variables
         z surplusPowerTotal;



variable surplusPower(t_a);
positive variable positiveSurplusPower(t_a);
positive variable negativeSurplusPower(t_a);
binary variable isPositive(t_a);
positive variable pvGenerationTotal(t_a);
positive variable electricalPowerTotal(t_a);


scalar
    minimalBufferTankTemperature_mHP /20/
    maximalBufferTankTemperature_mHP /22/
    minimalDHWTankTemperature_mHP /10/
    maximalDHWTankTemperature_mHP /45/
    efficiencyElectricHeating /0.98/
    efficiencyGasHeating /0.98/
    DensityOfWater / 988.02/
    DensityOfCement /2400/
    SpecificHeatCapacityOfCement /1000/
    SpecificHeatCapacityOfWater /4181/
    timeSlotForEndTemperature /287/
    timeResolution / 300/;
       


*Setting upper and lower limits for the temperature levels of the storages
temperature_level_DHWTank_mHP.lo(t_a, household_mHP) = minimalDHWTankTemperature_mHP;
temperature_level_DHWTank_mHP.up(t_a, household_mHP) = maximalDHWTankTemperature_mHP;

temperature_level_bufferTank_mHP.lo(t_a, household_mHP) = minimalBufferTankTemperature_mHP;
temperature_level_bufferTank_mHP.up(t_a, household_mHP) = maximalBufferTankTemperature_mHP;

*Fixing the inital temperature levels of the storages
temperature_level_DHWTank_mHP.fx("1", household_mHP) = initalDHWTankTemperature_mHP (household_mHP);
temperature_level_bufferTank_mHP.fx("1", household_mHP) = initalBufferStorageTemperature_mHP(household_mHP);



*Definition of the equations

Equations
eq_mHP_temperature_level_bufferTank_mHP(t_a, household_mHP)     hot water tank temperature
eq_mHP_temperature_level_DHWTank_mHP(t_a, household_mHP)     hot water tank temperature
eq_mHP_onlyOneTankAtOneTime (t_a, household_mHP)         
eq_mHP_onlyXorY_1 (t_a, household_mHP)
eq_mHP_onlyXorY_2 (t_a, household_mHP)
eq_fobj
eq_mHP_minimalEndTemperature_bufferTank(t_a, household_mHP)
eq_mHP_maximalEndTemperature_bufferTank (t_a, household_mHP)
eq_mHP_minimalEndTemperature_dhwTank(t_a, household_mHP)
eq_mHP_maximalEndTemperature_dhwTank (t_a, household_mHP)
eq_pvGenerationTotal(t_a)
eq_surplusPower(t_a)
eq_electricalPowerTotal(t_a)
e1(t_a), e2(t_a) ,e3(t_a)
eq_mHP_minimalCombinedModulation (t_a, household_mHP)
;


*Constraints for the minimal and maximal temperature at the end of the optimization horizon

eq_mHP_minimalEndTemperature_bufferTank(t_a, household_mHP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_mHP (t_a,household_mHP)
=g=
minimalBufferTankTemperature_mHP + 0.9;


eq_mHP_maximalEndTemperature_bufferTank(t_a, household_mHP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_bufferTank_mHP (t_a,household_mHP)
 =l=
 maximalBufferTankTemperature_mHP - 0.9;


eq_mHP_minimalEndTemperature_dhwTank(t_a, household_mHP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_mHP (t_a,household_mHP)
=g=
minimalDHWTankTemperature_mHP + 15;


eq_mHP_maximalEndTemperature_dhwTank(t_a, household_mHP) $(ord(t_a)=timeSlotForEndTemperature + 1)..
temperature_level_DHWTank_mHP (t_a,household_mHP)
 =l=
 maximalDHWTankTemperature_mHP - 15;


*Definition of the energetic difference equation for the temperature of the buffer storage
eq_mHP_temperature_level_bufferTank_mHP(t_a, household_mHP) $(ord(t_a)>1)..
temperature_level_bufferTank_mHP(t_a, household_mHP)
=e=
temperature_level_bufferTank_mHP(t_a-1, household_mHP) + ((x_mHP(t_a, household_mHP) * electricalPower_mHP(household_mHP) * timeResolution *  cop_BufferTank_mHP(t_a, household_mHP) - demand_heating_mHP(t_a, household_mHP)* timeResolution  - standingLoss_BufferTank_mHP(household_mHP) * timeResolution) /((volumeOfBufferTank_mHP(household_mHP)/1000) * DensityOfCement * SpecificHeatCapacityOfCement));


*Definition of the energetic difference equation for the temperature of the hot water tank (from an energy point of view equivalent to having the volume as the variable instead of the temperature)
eq_mHP_temperature_level_DHWTank_mHP(t_a, household_mHP) $(ord(t_a)>1)..
temperature_level_DHWTank_mHP(t_a, household_mHP)
=e=
temperature_level_DHWTank_mHP (t_a-1, household_mHP) + ((y_mHP(t_a, household_mHP) * electricalPower_mHP(household_mHP) * timeResolution * cop_DHWTank_mHP(t_a, household_mHP)     - demand_DHW_mHP(t_a, household_mHP) * timeResolution - standingLoss_DHWTank_mHP(household_mHP) * timeResolution) /((volumeOfDHWTank_mHP(household_mHP)/1000) * DensityOfWater * SpecificHeatCapacityOfWater));





*Constraints ensure that only one storage is heated up in each time slot
eq_mHP_onlyOneTankAtOneTime(t_a, household_mHP)..
y_mHP(t_a, household_mHP) + x_mHP(t_a, household_mHP) =l= 1;
eq_mHP_onlyXorY_1(t_a,household_mHP).. x_mHP(t_a,household_mHP)    =l= xAct_mHP(t_a,household_mHP);    
eq_mHP_onlyXorY_2(t_a,household_mHP).. y_mHP(t_a,household_mHP)    =l= (1-xAct_mHP(t_a,household_mHP));


*Constraints ensure a minimal modulation degree of the modulating heat pump such that it is not permanetly switched on and off
eq_mHP_minimalCombinedModulation (t_a, household_mHP)..
y_mHP(t_a, household_mHP) + x_mHP(t_a, household_mHP) =g= 0.1;


* Maximal values for the decision variables x and y is 1.0
x_mHP.up(t_a, household_mHP) = 1;
y_mHP.up(t_a, household_mHP) = 1;


*Equations for calculating the total PV generation, the surplus power and the total electrical power
eq_pvGenerationTotal(t_a).. pvGenerationTotal(t_a) =e= sum(household_mHP, (pv_Nominal_mHP(t_a, household_mHP) * pv_Peak_mHP(household_mHP)));
eq_surplusPower(t_a).. surplusPower(t_a) =e= pvGenerationTotal(t_a) - electricalPowerTotal(t_a);
eq_electricalPowerTotal(t_a)..electricalPowerTotal(t_a) =e= sum(household_mHP, (x_mHP(t_a, household_mHP)* electricalPower_mHP(household_mHP) + y_mHP(t_a,household_mHP) * electricalPower_mHP(household_mHP) + demand_electrical_mHP(t_a, household_mHP)));


e1(t_a).. surplusPower(t_a) =e= positiveSurplusPower(t_a) - negativeSurplusPower(t_a);
e2(t_a).. positiveSurplusPower(t_a) =l= bigMpos(t_a) * isPositive(t_a);
e3(t_a).. negativeSurplusPower(t_a) =l= bigMneg(t_a) * (1-isPositive(t_a));


*Objective function: minimize sum of surplus power over every time slot --> minimize surplus energy
eq_fobj ..
z
=e=
sum((t_a), positiveSurplusPower(t_a) );




*Big-M approach for ensuring that only the positive surplus power is minimized 

parameter       
       bigMpos(t_a)
       bigMneg(t_a)  ;
       


bigMpos(t_a) = sum(household_mHP, (pv_Nominal_mHP(t_a, household_mHP) * pv_Peak_mHP(household_mHP)));
bigMneg(t_a) = sum(household_mHP, (electricalPower_mHP(household_mHP) + electricalPower_mHP(household_mHP) + demand_electrical_mHP(t_a, household_mHP)));


*Defintion of the model that includes all equations
Model
     heating_scheduling /all/ ;


*Specifying the MIP gaps for the optimizations
heating_scheduling.optfile = 1;
heating_scheduling.optcr=option_relativeGap;
heating_scheduling.optca=option_absoluteGap;
heating_scheduling.Reslim=option_timeLimit;






Solve heating_scheduling using mip minimizing z;

*Calculating the time for the optimization
time_comp = TimeComp;
time_exec = TimeExec;
time_elapsed = TimeElapsed;

*Converting the sum of surplus power into surplus energy in kWh
result_in_kWh   = z.L * timeResolution / 3600000 ;








